﻿using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;

namespace Matrix
{
    public partial class MatrixControl : UserControl, INotifyPropertyChanged
    {
        // Änderungsbenachrichtigung
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged([CallerMemberName] string PropertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(PropertyName));
        }

        // Eigenschaften
        public int RowCount
        {
            get
            {
                return _RowCount;
            }
            private set
            {
                _RowCount = value;
                OnPropertyChanged();
            }
        }
        public int ColumnCount
        {
            get
            {
                return _ColumnCount;
            }
            private set
            {
                _ColumnCount = value;
                OnPropertyChanged();
            }
        }
        public double RowHeaderWidth
        {
            get { return (double)GetValue(RowHeaderWidthProperty); }
            set { SetValue(RowHeaderWidthProperty, value); }
        }
        public double ColumnHeaderHeight
        {
            get { return (double)GetValue(ColumnHeaderHeightProperty); }
            set { SetValue(ColumnHeaderHeightProperty, value); }
        }
        public double RowHeight
        {
            get { return (double)GetValue(RowHeightProperty); }
            set { SetValue(RowHeightProperty, value); }
        }
        public double ColumnWidth
        {
            get { return (double)GetValue(ColumnWidthProperty); }
            set { SetValue(ColumnWidthProperty, value); }
        }
        public ICollection RowHeadersSource
        {
            get { return (ICollection)GetValue(RowHeaderSourceProperty); }
            set { SetValue(RowHeaderSourceProperty, value); }
        }
        public ICollection ColumnHeadersSource
        {
            get { return (ICollection)GetValue(ColumnHeadersSourceProperty); }
            set { SetValue(ColumnHeadersSourceProperty, value); }
        }
        public ICollection ElementsSource
        {
            get { return (ICollection)GetValue(ElementsSourceProperty); }
            set { SetValue(ElementsSourceProperty, value); }
        }
        public DataTemplate RowHeaderTemplate
        {
            get { return (DataTemplate)GetValue(RowHeaderTemplateProperty); }
            set { SetValue(RowHeaderTemplateProperty, value); }
        }
        public DataTemplate ColumnHeaderTemplate
        {
            get { return (DataTemplate)GetValue(ColumnHeaderTemplateProperty); }
            set { SetValue(ColumnHeaderTemplateProperty, value); }
        }
        public DataTemplate ElementTemplate
        {
            get { return (DataTemplate)GetValue(ElementTemplateProperty); }
            set { SetValue(ElementTemplateProperty, value); }
        }
        
        public static readonly DependencyProperty RowHeaderWidthProperty =
            DependencyProperty.Register("RowHeaderWidth", typeof(double), typeof(MatrixControl), new PropertyMetadata(0.0));
        public static readonly DependencyProperty ColumnHeaderHeightProperty =
            DependencyProperty.Register("ColumnHeaderHeight", typeof(double), typeof(MatrixControl), new PropertyMetadata(0.0));
        public static readonly DependencyProperty RowHeightProperty =
            DependencyProperty.Register("RowHeight", typeof(double), typeof(MatrixControl), new PropertyMetadata(0.0));
        public static readonly DependencyProperty ColumnWidthProperty =
            DependencyProperty.Register("ColumnWidth", typeof(double), typeof(MatrixControl), new PropertyMetadata(0.0));
        public static readonly DependencyProperty RowHeaderSourceProperty =
            DependencyProperty.Register("RowHeadersSource", typeof(ICollection), typeof(MatrixControl), new PropertyMetadata(null, RowHeadersSourceChanged));
        public static readonly DependencyProperty ColumnHeadersSourceProperty =
            DependencyProperty.Register("ColumnHeadersSource", typeof(ICollection), typeof(MatrixControl), new PropertyMetadata(null, ColumnHeadersSourceChanged));
        public static readonly DependencyProperty ElementsSourceProperty =
            DependencyProperty.Register("ElementsSource", typeof(ICollection), typeof(MatrixControl), new PropertyMetadata(null));
        public static readonly DependencyProperty RowHeaderTemplateProperty =
            DependencyProperty.Register("RowHeaderTemplate", typeof(DataTemplate), typeof(MatrixControl), new PropertyMetadata(null));
        public static readonly DependencyProperty ColumnHeaderTemplateProperty =
            DependencyProperty.Register("ColumnHeaderTemplate", typeof(DataTemplate), typeof(MatrixControl), new PropertyMetadata(null));
        public static readonly DependencyProperty ElementTemplateProperty =
            DependencyProperty.Register("ElementTemplate", typeof(DataTemplate), typeof(MatrixControl), new PropertyMetadata(null));

        private int _RowCount;
        private int _ColumnCount;

        // Konstruktor
        public MatrixControl()
        {
            InitializeComponent();
        }

        // Methoden
        private void RowHeaderScrollViewer_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if(e.VerticalChange != 0)
            {
                ElementsScrollViewer.ScrollToVerticalOffset(e.VerticalOffset);
                RowScrollBar.Value = e.VerticalOffset;
            }
        }
        private void ColumnHeadersScrollViewer_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (e.HorizontalChange != 0)
            {
                ElementsScrollViewer.ScrollToHorizontalOffset(e.HorizontalOffset);
                ColumnScrollBar.Value = e.HorizontalOffset;
            }
        }
        private void ElementsScrollViewer_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (e.VerticalChange != 0)
            {
                RowHeaderScrollViewer.ScrollToVerticalOffset(e.VerticalOffset);
                RowScrollBar.Value = e.VerticalOffset;
            }
            if (e.HorizontalChange != 0)
            {
                ColumnHeadersScrollViewer.ScrollToHorizontalOffset(e.HorizontalOffset);
                ColumnScrollBar.Value = e.HorizontalOffset;
            }
        }
        private void RowScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            if (e.NewValue != 0)
            {
                ElementsScrollViewer.ScrollToVerticalOffset(e.NewValue);
                RowHeaderScrollViewer.ScrollToVerticalOffset(e.NewValue);
            }
        }
        private void ColumnScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            if (e.NewValue != 0)
            {
                ElementsScrollViewer.ScrollToHorizontalOffset(e.NewValue);
                ColumnHeadersScrollViewer.ScrollToHorizontalOffset(e.NewValue);
            }
        }

        private void RowHeaders_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            RowCount = (sender as ICollection).Count;
        }
        private void ColumnHeaders_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            ColumnCount = (sender as ICollection).Count;
        }

        private static void RowHeadersSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MatrixControl m = d as MatrixControl;

            // CollectionChanged (de)registrieren, wenn die Collections dies unterstützen
            INotifyCollectionChanged o = e.OldValue as INotifyCollectionChanged;
            if (o != null)
                o.CollectionChanged -= m.RowHeaders_CollectionChanged;
            INotifyCollectionChanged c = e.NewValue as INotifyCollectionChanged;
            if(c != null)
                c.CollectionChanged += m.RowHeaders_CollectionChanged;

            // Zeilenanzahl neu berechnen lassen
            m.RowHeaders_CollectionChanged(c, null);
        }
        private static void ColumnHeadersSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MatrixControl m = d as MatrixControl;

            // CollectionChanged (de)registrieren, wenn die Collections dies unterstützen
            INotifyCollectionChanged o = e.OldValue as INotifyCollectionChanged;
            if (o != null)
                o.CollectionChanged -= m.ColumnHeaders_CollectionChanged;
            INotifyCollectionChanged c = e.NewValue as INotifyCollectionChanged;
            if (c != null)
                c.CollectionChanged += m.ColumnHeaders_CollectionChanged;

            // Spaltenanzahl neu berechnen lassen
            m.ColumnHeaders_CollectionChanged(c, null);
        }

        private void ColumnScrollBar_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            ColumnScrollBar.LargeChange = e.NewSize.Width / 20;
            ColumnScrollBar.SmallChange = e.NewSize.Width / 50;
        }

        private void RowScrollBar_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            RowScrollBar.LargeChange = e.NewSize.Height / 20;
            RowScrollBar.SmallChange = e.NewSize.Height / 50;
        }
    }
}
